/* $Header */

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "exports.h"
#include "cwstart.h"

LIBMAIN

void DLL 
_init()
{
	LibExport( "BOOL TagTrackCwd   BOOL");
	LibExport( "INT  TagDirHandler LONG LONG");
	LibExport( "BOOL SysTrackCwd   BOOL");
	LibExport( "INT  SysDirHandler LONG LONG");
	LibExport( "     TagQuery");
}

/*
 * This event handler is called whenever a file buffer becomes current.
 * It determines the directory in which the newly current file is
 * located and makes Codewright's idea of the current tag file relative
 * to that directory.
 *
 * Only the basename and extension of the tagfile, as specified in the
 * TagSetFile command, will be used.  Directory path information will
 * be discarded.
 *
 * The purpose of this handler is to facilitate the use of Codewright
 * for multiple projects.  By default, Codewright assumes only one tag
 * file per session.  With this handler, the user need not bother about
 * setting the tagfile name by hand when changing projects.
 */
 
int DLL TagDirHandler (long event, long data);

	static HEVENT eHandle = NULL;
	
void DLL
TagQuery(void)
{
	MsgPrintf(3, "%lx", (unsigned long)eHandle);
}

BOOL DLL
TagTrackCwd (BOOL turnon)
{
	
	if (turnon)
	{
		TagDirHandler(0, 0);	// start in right away
		if (eHandle = EventRegister (EVENT_BUFFER_CURRENT, EVENT_NORMAL,
			"TagDirHandler"))
			return(TRUE);				// registration succeeded
		return(FALSE);
	}
	else if (eHandle)
	{
		if (!EventDeregister (eHandle))	// successfully deregistered
			return(TRUE);
		else							// deregistration failed
			return(FALSE);
	}
}

/*
 * The event handler.  Returns TRUE if and only if it reset the
 * tag file name.
 */
 

int DLL
TagDirHandler (long event, long data)
{
	LPSTR   tagFile  = NULL;
	LPSTR   fileNam;
	LPSTR   drive;  
	LPSTR   path;   
	LPSTR   base;   
	LPSTR   ext;    
	HBUFFER curBuf; 
	
	/* do nothing if there's no current user buffer	*/
	if (!(curBuf = BufQCurrentBuffer()) || BufQIsSystem(curBuf))
		return((long)FALSE);

	if ((fileNam = BufQFilename()) && *fileNam)
	{
		if (!(fileNam = StrNormalizePath(fileNam))) /* can't get file's directory */
			return((long)FALSE);
		if (!StrSplitpath(fileNam, &drive, &path, NULL, NULL))
			return((long)FALSE);
		if (tagFile = MemAlloc(strlen(drive) + strlen(path) + 2))
		{
			strcpy(tagFile, drive);
			strcat(tagFile, ":");
			strcat(tagFile, path);
		}
		StrFree(drive);
		StrFree(path);
		StrFree(fileNam);
	}

	/* reuse fileNam */
	if ((fileNam = (LPSTR)LibFunctionExec("TagQFilename")) && *fileNam)
	{
		if (!(fileNam = StrNormalizePath(fileNam)))
			return((long)FALSE);
		if (!StrSplitpath(fileNam, NULL, NULL, &base, &ext))
			return((long)FALSE);
		if (tagFile[strlen(tagFile)-1] != '\\')
			tagFile = StrAppend(tagFile, "\\");
		tagFile = StrAppend(tagFile, base);
		tagFile = StrAppend(tagFile, ext);
		TagSetFile(tagFile);
		StrFree(base);
		StrFree(ext);
		StrFree(fileNam);
		StrFree(tagFile);
		return((long)TRUE);
	}
	return((long)FALSE);
}

/*
 * This event handler is called whenever a file buffer becomes current.
 * It determines the directory in which the newly current file is
 * located and makes sets Codewright's idea of the current directory
 * to that directory.
 *
 * The purpose of this handler is to make it easier to edit files which
 * are in the same directory because they are related.  For example,
 * many programmers dedicate one or more directories to one project's
 * files.  Using this handler, it's easy to open another file in the
 * same directory as the one you're currently editing.
 *
 * Use SysTrackCwd to install or remove the handler.  It will install
 * (and call) the handler if its argument is TRUE; otherwise it will
 * attempt to remove it.  It returns TRUE if and only if the operation
 * was successful.
 *
 * The return value from the handler itself is meant to be cast to
 * BOOL.  It is TRUE if and only if the directory was successfully changed.
 */
 

int DLL SysDirHandler (long event, long data);
	
BOOL DLL
SysTrackCwd(BOOL turnon)
{
	static HEVENT eHandle = NULL;
	
	if (turnon)
	{
		SysDirHandler(0, 0);			/* start in right away			*/	
		if (eHandle = EventRegister (EVENT_BUFFER_CURRENT, EVENT_NORMAL,
			"SysDirHandler"))
			return(TRUE);				/* registration succeeded		*/
	}
	else if (eHandle)
	{
		if (!EventDeregister (eHandle))	/* successfully deregistered	*/
			return(TRUE);
	}
 	return(FALSE);
}


int DLL
SysDirHandler(LONG event, LONG data)
{
	LPSTR   fileName; 
	LPSTR   newWd;    
	LPSTR   drive;
	LPSTR   path;
	HBUFFER curBuf;   
	int     succeeded;
	
	/* do nothing if there's no current user buffer	*/
	if (!(curBuf = BufQCurrentBuffer()) || BufQIsSystem(curBuf))
		return((long)FALSE);

	if ((fileName = BufQFilename()) && *fileName)
	{
		if (!(fileName = StrNormalizePath(fileName))) /* can't get directory */
			return((long)FALSE);
		if (!StrSplitpath(fileName, &drive, &path, NULL, NULL))
		{
			StrFree(fileName);
			return((long)FALSE);
		}
		if (newWd = MemAlloc(strlen(drive) + strlen(path) + 2))
		{
			strcpy(newWd, drive);
			strcat(newWd, ":");
			strcat(newWd, path);
			succeeded = SysSetCwd(newWd);
			StrFree(drive);
			StrFree(path);
			StrFree(fileName);
			StrFree(newWd);
			return(succeeded);
		}
	}
	return(FALSE);
}

